function HTphase1D(algtype,signaltype)
%HTphase1D serves as a test of heavy tailed signal recovery in the 
% compressive sensing scenario.  It cycles through the number of 
% measurements m for a fixed signal dimension N.
%
%
%SYNTAX:
%HTphas1D(algtype,signaltype)
%
%INPUTS:
% algtype - which algorithm to run
% - 'EMBGAMP'
% - 'EMGMAMP'
% - 'EMGMAMPMOS'
% - 'SL0' Smoothed L0 norm
% - 'BCS' Bayesian Compressive Sensing
% - 'SBL' Sparse bayesian laearning via TMSBL
% - 'LASSO' LASSO via SPGL-1
% - 'OMP' Orhtongonal Matching Pursuit via Solve OMP
% - 'SP' Subspace Pursuit
% signaltype - which type of signal
% -'LOGN' logn-normal
% -'ST'   Student's-t
%
%Outputs
%The dated .mat file will containing the nmse for each gridpoint.  
%It also contains the CPU time for the algorithm
%
%Coded by Jeremy Vila
%The Ohio State University
%vila.2@osu.edu
%Date 7-01-13
%Version 1.0

tstart2 = tic;
%define seed based on internal clock
NewStream = RandStream.create('mt19937ar', 'Seed', sum(10000*clock));
% RandStream.setDefaultStream(NewStream);
% savedState = NewStream.State;

%Handle random seed
if verLessThan('matlab','7.14')
  RandStream.setDefaultStream(NewStream);
else
  RandStream.setGlobalStream(NewStream);
end;

%This is needed for calculation of LASSO PTC.
%Define cdf/pdf for Gaussian
normal_cdf = @(x) 1/2*(1 + erf(x/sqrt(2)));

%Define density of Gaussian
normal_pdf = @(x) 1/sqrt(2*pi)*exp(-x.^2/2);

%size of signal
N = 1000;
%SNR in dB
SNRdB = 25;
%amount of undersamping
rows = 300:20:600;

%initialize the nmse and time vectors
nmse = zeros(1,length(rows));
time = zeros(1,length(rows));
for i = 1:16
tstart = tic;

%Define signal
M = rows(i);

%Calculate LASSO PTC
delta = M/N;
temp = linspace(0,100,1024);
rho_SE = (1 - (2/delta)*((1+temp.^2).*normal_cdf(-temp)-temp.*normal_pdf(temp)))...
    ./(1 + temp.^2 - 2*((1+temp.^2).*normal_cdf(-temp)-temp.*normal_pdf(temp)));
rho = max(rho_SE);

%A matrix, Gaussian
Amat = randn(M,N);

%Normalize the columns
columnNorms = sqrt(diag(Amat'*Amat));
Amat = Amat*diag(1 ./ columnNorms); %unit norm columns

%Rename A
A = MatrixLinTrans(Amat);

%Generate signal
%Student's T
if strcmp(signaltype,'ST')
    x = randcs(N, 1.67 , 1 , 't')';
%Log Normal
elseif strcmp(signaltype,'LOGN');
    x = randcs(N, 0, 1, 'logn')';
end

%Form noiseless measurements
ztrue = A.mult(x);

%Calcualte noise variance based on SNR and signal
muw(i) = norm(A.mult(x))^2/M*10^(-SNRdB/10);

%Compute noisy output
y = ztrue + sqrt(muw(i))*(randn(M,1));

%EMBGAMP
if strcmp(algtype,'EMBGAMP')
    xhat = EMBGAMP(y, A);
%EMGMAMP
elseif strcmp(algtype,'EMGMAMP')
    optEM.heavy_tailed = true;
    xhat = EMGMAMP(y, A, optEM);
%EMGMAMPMOS
elseif strcmp(algtype,'EMGMAMPMOS')
    optEM.heavy_tailed = true;
    xhat = EMGMAMPMOS(y, A, optEM);
%SL0 algorithm
elseif strcmp(algtype,'SL0')
    sigma_min = 0.00004;
    xhat = SL0(Amat, y, sigma_min);
%Bayesian Compressive Sensing
elseif strcmp(algtype,'BCS')
    xhat = mt_CS(Amat,y,100/var(y),1,1e-8);
%Sparse Bayesian Learning via TMSBL
elseif strcmp(algtype,'SBL')
    if SNRdB < 22
        xhat = TMSBL(Amat, y,'noise','mild','PRINT',0);
    else
        xhat = TMSBL(Amat, y,'noise','small','PRINT',0); 
    end
elseif strcmp(algtype,'LASSO')
%LASSO via SPGL-1
    options.verbosity = 0;
    %Create grid of "lambda values"
    scale = [0.6:0.2:1.4];
    for v=1:length(scale),      % find nmse-optimal error constraint
         xhat = spgl1(Amat,y,0,sqrt(scale(v)*muw(i)*M),[],options);
         %Debias the estimates
         A_debiased = Amat(:,(xhat~=0));
         xhat_debiased = zeros(N,1);
         xhat_debiased((xhat~=0)) = A_debiased\y;
         nmse_x_test(v) = norm(xhat_debiased-x)^2/norm(x)^2;
    end
%OMP
elseif strcmp(algtype,'OMP')
    %Generate grid of postulated K based on LASSO PTC
    iters = floor(linspace(1,floor(rho*M),10));
    %Perform OMP for this grid and save the nmse for each test
    for j = 1:length(iters)
        xhat = SolveOMP(Amat, y, N,iters(j));
        nmse_x_test(j) = norm(xhat-x)^2/norm(x)^2;
    end  
%Subspace pursuit
elseif strcmp(algtype,'SP')
    %Generate grid of postulated K based on LASSO PTC
    iters = floor(linspace(1,floor(rho*M),10));
    %Perform SP for this grid and save the nmse for each test
    for j = 1:length(iters)
        tmp = CSRec_SP(iters(j), Amat, y);
        xhat = tmp.x_hat;
        nmse_x_test(j) = norm(xhat-x)^2/norm(x)^2;
    end
end

%Get the best NMSE for LASSO,SP,or OMP
if (strcmp(algtype,'LASSO') || strcmp(algtype,'SP') || strcmp(algtype,'OMP'))
    [tmp,indx] = min(nmse_x_test);
else
    tmp = (norm(xhat-x)^2)/(norm(x)^2);
end

%get nmse and time
nmse(i) = tmp;
time(i) = toc(tstart);

end;

%Save the results
date = datestr(now, 'ddmmmyyyy_HHMMSSFFF');
savefile = [strcat('HTphas1D_',algtype,'_',signaltype,'_') date '.mat'];

save(savefile, 'nmse', 'time', 'rows')

toc(tstart2)
return