function MATCMPPTC(griddens, mattyp, BRrate)
%MATCMPPTC serves as a test of the sparse Bernoulli Gaussian signal
%recovery. It cycles through the undersampling ratio (m/n),delta, and the
%normalized sparsity for different specified sensing operators.
%
%
%SYNTAX:
%MATCMPPTC(griddens, mattyp)
%
%INPUTS:
%griddens           Total number of rows and columns of lmmse grid
%mattyp             Matrix type
%   -GAUS           Elements are i.i.d. Gaussian.
%   -RAD            Elements are i.i.d. Rademacher with prob = 0.5.
%   -UNIF           Elements are i.i.d. Uniform on [-0.5,0.5].
%   -CAUCHY         Elements are i.i.d. Cauchy with location = 0 and scale
%                   = 1.
%   -BERN           Elements are i.i.d. Bernoulli {0,1} with probability of
%                   "active" (=1) of 0.8;
%   -DCT            Elements are from oversampled DCT operator (random row)
%BRrate - If using BR matrices, the user must specify the rate parameter
%for percentage of active elements per column.
%
%Outputs
%The dated .mat file will contain a matrix of nmse at each gridpoint.
%
%Coded by Jeremy Vila
%The Ohio State University
%vila.2@osu.edu
%Date 7-01-13
%Version 1.0

tic

%define seed based on internal clock
NewStream = RandStream.create('mt19937ar', 'Seed', sum(10000*clock));
% RandStream.setDefaultStream(NewStream);
% savedState = NewStream.State;

%Handle random seed
if verLessThan('matlab','7.14')
  RandStream.setDefaultStream(NewStream);
else
  RandStream.setGlobalStream(NewStream);
end;

N = 1000;

Q = griddens; %density of grid
gamma = linspace(.05, .95, Q);
delta = linspace(.05, .95, Q);
N_gamma = length(gamma);
N_delta = length(delta);

window = 0;

nmse = ones(griddens,griddens);

%perform EMGMAMP for each point in 2D phase plane
for d = 1:N_delta
for g = 1:N_gamma
   

M = round(N*delta(d));
K = floor(gamma(g)*M);

%Avoid zero K
if K == 0
    K = 1;
end;

%Gaussian matrix
if strcmp(mattyp,'GAUS')
    Amat = randn(M,N);
    
    A = MatrixLinTrans(Amat);
%Rademacher matrix
elseif strcmp(mattyp,'RAD')
    Amat = sign(randn(M,N));
    
    A = MatrixLinTrans(Amat);
%Uniform matrix
elseif strcmp(mattyp,'UNIF')
    Amat = rand(M,N) - 0.5;
    
    A = MatrixLinTrans(Amat);
%Cauchy matrix
elseif strcmp(mattyp,'CAUCHY')
    Amat = cauchyrnd(0,1,[M,N]);
    
    A = MatrixLinTrans(Amat);
%Bernoulli matrix
elseif strcmp(mattyp,'BERN')
    
act_elem = ceil(0.15*M);
Amat = [ones(act_elem,N); zeros(M-act_elem,N)];

for n = 1:N
    Amat(:,n) = Amat(randperm(M),n);
    
    %get indexes of all other elements
    idx = setdiff(1:N,n);
    %replicate the indexe's column for a comparison test
    dummy = repmat(Amat(:,n),1,N-1);
    
    %Compare if one column is the same as another, if so change
    while(any(sum(dummy == Amat(:,idx)) == M))
        Amat(:,n) = Amat(randperm(M),n);
        dummy = repmat(Amat(:,n),1,N-1);
    end
end

A = MatrixLinTrans(Amat);

%Bernoulli Rademacher matrix
elseif strcmp(mattyp,'BR')
    
act_elem = ceil(BRrate*M);
Amat = [sign(randn(act_elem,N)); zeros(M-act_elem,N)];

for n = 1:N
    Amat(:,n) = Amat(randperm(M),n);
    
    %get indexes of all other elements
    idx = setdiff(1:N,n);
    %replicate the indexe's column for a comparison test
    dummy = repmat(Amat(:,n),1,N-1);
    
    %Compare if one column is the same as another
    while(any(sum(dummy == Amat(:,idx)) == M))
        Amat(:,n) = Amat(randperm(M),n);
        dummy = repmat(Amat(:,n),1,N-1);
    end
end

A = MatrixLinTrans(Amat);

%Use DCT operator
elseif strcmp(mattyp,'DCT');
    A = DCTLinTrans(N,N,1);

    A.ySamplesRandom(M);
end

%Determine true bits
truebits = false(N,1);
truebits(1:K) = true; %which bits are on

%Compute true input vector
x = randn(N,1).* truebits;
x = x(randperm(length(x)));
ztrue = A.mult(x);

%Output channel- we assume AWGN with mean thetahat and variance muw
SNRdB = Inf;
muw = norm(ztrue)^2/M*10^(-SNRdB/10);

%Compute noisy output
y = ztrue + sqrt(muw)*(randn(M,1));

%Perform EMGMAMP
optGAMP = GampOpt();
optGAMP.nit = 100;
optGAMP.adaptStep = false;
optGAMP.pvarMin = 1e-6;
optGAMP.xvarMin = 1e-6;
optGAMP.pvarStep = false;
optGAMP.stepMin = 0.5;
optGAMP.tol = 1e-6;

optEM.EMtol = 1e-6;
optEM.maxEMiter = 50;
optEM.heavy_tailed = false;
[xhat, ~, ~, ~] = EMGMAMP(y, A, optEM, optGAMP);


nmse(g,d) = (norm(xhat-x)^2)/(norm(x)^2);

if 10*log10(nmse(g,d)) > -5
    window = window + 1;
end;

if window > 2
    break
end


end;
window = 0;
end;
toc

% Save the results
date = datestr(now, 'ddmmmyyyy_HHMMSSFFF');
if ~strcmp(mattyp,'BR')
    savefile = [strcat('MATCMP_', mattyp,'_') num2str(date) '.mat'];
else
    savefile = [strcat('MATCMP_', mattyp,'_',num2str(BRrate),'_') num2str(date) '.mat'];   
end

save(savefile, 'nmse','mattyp')
clear all