function PTC(griddens, algtype, signaltype)
%phase 2D is a function to generate PTCs.  It saves the nmse for each point
%on a griddens x griddens grid from (0.05,0.95) x (0.05,0.95).  
%
%
%SYNTAX:
%PTC(griddens, algtype, signaltype)
%
%INPUTS:
%griddens           Total number of rows and columns of lmmse grid
%signaltype         type of signal
% -BR               Bernoulli Rademacher
% -BG               Bernoulli Gaussian
% -B1               Bernoulli
%algtype
% algtype - which algorithm to run
% - 'DMMAMP' Donoho Maleki and Montanari AMP
% - 'EMBGAMP'
% - 'EMGMAMP'
% - 'EMGMAMPMOS'
% - 'BCS' Bayesian Compressive Sensing
% - 'OMP' Orhtongonal Matching Pursuit via Solve OMP
% - 'SP' Subspace Pursuit
%Outputs
%The dated .mat file will contain a matrix of nmse for each gridpoint.
%
%Coded by Jeremy Vila
%The Ohio State University
%vila.2@osu.edu
%Date 7-01-13
%Version 1.3

tic

%define seed based on internal clock
NewStream = RandStream.create('mt19937ar', 'Seed', sum(10000*clock));
% RandStream.setDefaultStream(NewStream);
% savedState = NewStream.State;

%Handle random seed
if verLessThan('matlab','7.14')
  RandStream.setDefaultStream(NewStream);
else
  RandStream.setGlobalStream(NewStream);
end;

N = 1000;

Q = griddens; %density of grid
gamma = linspace(.05, .95, Q);
delta = linspace(.05, .95, Q);
N_gamma = length(gamma);
N_delta = length(delta);

nmse = ones(Q,Q);

window = 0;

for d = 1:N_delta
for g = 1:N_gamma
   
M = round(N*delta(d));
K = floor(gamma(g)*M);

%Avoid zero K
if K == 0
    K = 1;
end

Amat = randn(M,N);

%Normalize the columns
columnNorms = sqrt(diag(Amat'*Amat));
Amat = Amat*diag(1 ./ columnNorms); %unit norm columns

A = MatrixLinTrans(Amat);

%Determine true bits
truebits = false(N,1);
truebits(1:K) = true; %which bits are on

%Compute true input vector
if strcmp(signaltype,'BG')
    x = randn(N,1).* truebits;
elseif strcmp(signaltype,'BR')
    x = sign(randn(N,1)).* truebits;
elseif strcmp(signaltype,'B1')
    x = ones(N,1).* truebits;
end;
x = x(randperm(length(x)));
ztrue = Amat*x;

SNR = Inf;
%Output channel- we assume AWGN with mean thetahat and variance muw
muw = norm(ztrue)^2/M*10^(-SNR/10);

%Compute noisy output
y = ztrue + sqrt(muw)*(randn(M,1));

%Use DMMAMP
if strcmp(algtype,'DMMAMP')
%Set options for Rangan GAMP
opt = GampOpt(); %initialize the options object
opt.nit = 2500;
opt.adaptStep = false;
opt.stepMax = 0.6;
opt.bbStep = false;
opt.verbose = false;
opt.removeMean = false;
opt.pvarMin = 0;
opt.xvarMin = 0;
opt.xhat0 = zeros(N,1);
opt.xhat0 = 1e-1*ones(N,1);
opt.tol = 1e-6;

%Define cdf/pdf for Gaussian
normal_cdf = @(x) 1/2*(1 + erf(x/sqrt(2)));

%Define density of Gaussian
normal_pdf = @(x) 1/sqrt(2*pi)*exp(-x.^2/2);

%Define an alpha grid
alpha_grid = linspace(0.001,5,1024);

delta_grid = 2*normal_pdf(alpha_grid)./...
    (alpha_grid + 2*(normal_pdf(alpha_grid) - alpha_grid .* normal_cdf(-1*alpha_grid)));

[~,b] = min(abs(delta_grid - M/N));

alpha = alpha_grid(b);

%Create the input estimator using soft threshold
inputEst = SoftThreshEstimIn(alpha);

%Output channel
outputEst = AwgnEstimOut(y, 0);

[xhat, ~, ~, ~,...
    ~, ~, ~] = ...
    gampEst(inputEst, outputEst, A, opt);

%Use EMBGAMP
elseif strcmp(algtype,'EMBGAMP')
optGAMP = GampOpt();
optGAMP.nit = 100;
optGAMP.adaptStep = false;
optGAMP.pvarMin = 1e-6;
optGAMP.xvarMin = 1e-6;
optGAMP.pvarStep = false;
optGAMP.stepMin = 0.5;
optGAMP.tol = 1e-6;

optEM.EMtol = 1e-6;
optEM.maxEMiter = 50;
optEM.heavy_tailed = false;

xhat = EMBGAMP(y, A, optEM, optGAMP);

%Use EMGMAMP
elseif strcmp(algtype,'EMGMAMP')
optGAMP = GampOpt();
optGAMP.nit = 100;
optGAMP.adaptStep = false;
optGAMP.pvarMin = 1e-6;
optGAMP.xvarMin = 1e-6;
optGAMP.pvarStep = false;
optGAMP.stepMin = 0.5;
optGAMP.tol = 1e-6;

optEM.EMtol = 1e-6;
optEM.maxEMiter = 50;
optEM.heavy_tailed = false;
    
xhat = EMGMAMP(y, A, optEM, optGAMP);

%Use EMGMAMPMOS
elseif strcmp(algtype,'EMGMAMPMOS')
optGAMP = GampOpt();
optGAMP.nit = 100;
optGAMP.adaptStep = false;
optGAMP.pvarMin = 1e-6;
optGAMP.xvarMin = 1e-6;
optGAMP.pvarStep = false;
optGAMP.stepMin = 0.5;
optGAMP.tol = 1e-6;

optEM.EMtol = 1e-6;
optEM.maxEMiter = 50;
optEM.heavy_tailed = false;

xhat = EMGMAMPMOS(y, A, optEM, optGAMP);

%Use BCS
elseif strcmp(algtype,'BCS')
    
xhat = mt_CS(Amat,y,1000000*var(y),1,1e-12);

%Use SP
elseif strcmp(algtype,'SP')
warning('off','all');
%Generate grid of postulated 
iters = floor(linspace(1,min(2*K,signalsize),20));
nmse_x_test = nan(1,length(iters));
%Perform OMP for this grid and save the nmse for each test
for j = 1:length(iters)
    tmp = CSRec_SP(iters(j), Amat, y);
    xhat = tmp.x_hat;
    nmse_x_test(j) = norm(xhat-x)^2/norm(x)^2;
end

%Use OMP
elseif strcmp(algtype,'OMP')
%Generate grid of postulated K
iters = floor(linspace(1,min(2*K,signalsize),20));
nmse_x_test = nan(1,length(iters));
%Perform OMP for this grid and save the nmse for each test
for j = 1:length(iters)
    xhat = SolveOMP(Amat, y, N, iters(j));
    nmse_x_test(j) = norm(xhat-x)^2/norm(x)^2;
end  

end

%Get the best NMSE for LASSO,SP,or OMP
if strcmp(algtype,'SP') || strcmp(algtype,'OMP')
    [nmse(g,d),indx] = min(nmse_x_test);
else
    nmse(g,d) = (norm(xhat-x)^2)/(norm(x)^2);
end

if 10*log10(nmse(g,d)) > -10
    window = window + 1;
end;

if window > 1
    break
end

end;
window = 0;
end;

toc

%save the nmse
date = datestr(now, 'ddmmmyyyy_HHMMSSFFF');
savefile = [strcat('phas2D_',signaltype,'_', algtype,'_') date '.mat'];

save(savefile, 'nmse')

clear all