function SNRphase1D(algtype)
%SNRphase1D serves as a test of the sparse Bernoulli Rademacher signal
%recovery. For a fixed undersampling ratio M/N, the SNR is then varied.
%
%
%SYNTAX:
%SNRphase1D(algtype)
%
%INPUTS:
%algtype         type of algorithm
% - 'EMBGAMP'
% - 'EMGMAMP'
% - 'EMGMAMPMOS'
% - 'SL0' Smoothed L0 norm
% - 'BCS' Bayesian Compressive Sensing
% - 'SBL' Sparse bayesian laearning via TMSBL
% - 'LASSO' LASSO via SPGL-1
% - 'OMP' Orhtongonal Matching Pursuit via Solve OMP
% - 'SP' Subspace Pursuit
%
%Outputs
%The dated .mat file will contain a vector of nmse at each gridpoint.  
%It also contains the SNR grid for reference.
%
%Coded by Jeremy Vila
%The Ohio State University
%vila.2@osu.edu
%Date 7-01-13
%Version 1.0

tstart = tic;
%define seed based on internal clock
NewStream = RandStream.create('mt19937ar', 'Seed', sum(10000*clock));
% RandStream.setDefaultStream(NewStream);
% savedState = NewStream.State;

%Handle random seed
if verLessThan('matlab','7.14')
  RandStream.setDefaultStream(NewStream);
else
  RandStream.setGlobalStream(NewStream);
end;

%Define cdf/pdf for Gaussian
normal_cdf = @(x) 1/2*(1 + erf(x/sqrt(2)));

%Define density of Gaussian
normal_pdf = @(x) 1/sqrt(2*pi)*exp(-x.^2/2);

%Form SNR grid
gridsize = 11;
SNR = linspace(10,50,gridsize);

%Problem dimensions
N = 1000;
M = 500;
K = 100;

delta = M/N;

for i = 1:gridsize

%A matrix, Gaussian
Amat = randn(M,N);

%%%%%%%%%%% 
%Explicit normalized A matrix
%Normalize the columns
columnNorms = sqrt(diag(Amat'*Amat));
Amat = Amat*diag(1 ./ columnNorms); %unit norm columns

%Rename A
A = MatrixLinTrans(Amat); 

% %Determine true bits
truebits = false(N,1);
truebits(1:K) = true; %which bits are on
% 
% %Compute true input vector
x = sign(randn(N,1)).* truebits;
x = x(randperm(length(x)));

%%%%%%%%%%
% A operator for explicit and GAMP fft
 ztrue = A.mult(x);
 
%Output channel- we assume AWGN with mean thetahat and variance muw
muw(i) = norm(A.mult(x))^2/M*10^(-SNR(i)/10);

%Compute noisy output
y = ztrue + sqrt(muw(i))*(randn(M,1));

optEM.heavy_tailed = false;
%EMBGAMP using sourceforge
if strcmp(algtype,'EMBGAMP')
    xhat = EMBGAMP(y, A, optEM);
%EMGMAMP using sourceforge
elseif strcmp(algtype,'EMGMAMP')
    xhat = EMGMAMP(y, A, optEM);
elseif strcmp(algtype,'EMGMAMPMOS')
    xhat = EMGMAMPMOS(y, A, optEM);
%SL0 algorithm
elseif strcmp(algtype,'SL0')
    %sigma_min = sqrt(muw(t))*3;
    sigma_min = 0.00004;
    xhat = SL0(Amat, y, sigma_min);
%Bayesian Compressive sensing algorithm BCS
elseif strcmp(algtype,'BCS')
    xhat = mt_CS(Amat,y,100/var(y),1,1e-8);
elseif strcmp(algtype,'SBL')
%Sparse Bayesian Learning programmed by Zhiling Zhang
    if SNR(i) < 22
        xhat = TMSBL(Amat, y,'noise','mild','PRINT',0);
    else
        xhat = TMSBL(Amat, y,'noise','small','PRINT',0); 
    end
elseif strcmp(algtype,'LASSO')
%LASSO via SPGL-1
    options.verbosity = 0;
    scale = [0.6:0.2:1.5];
    for v=1:length(scale),      % find nmse-optimal error constraint
     xhat = spgl1(Amat,y,0,sqrt(scale(v)*muw(i)*M),[],options);
     A_debiased = Amat(:,(xhat~=0));
     xhat_debiased = zeros(N,1);
     xhat_debiased((xhat~=0)) = A_debiased\y;
     nmse_x_test(v) = norm(xhat_debiased-x)^2/norm(x)^2;
    end
%ALPS
elseif strcmp(algtype,'OMP')
    iters = floor(linspace(1,2*K,10));
    for j = 1:length(iters)
        xhat = SolveOMP(Amat, y, N,iters(j));
        nmse_x_test(j) = norm(xhat-x)^2/norm(x)^2;
    end     
elseif strcmp(algtype,'SP')
    iters = floor(linspace(1,2*K,10));
    for j = 1:length(iters)
        tmp = CSRec_SP(iters(j), Amat, y);
        xhat = tmp.x_hat;
        nmse_x_test(j) = norm(xhat-x)^2/norm(x)^2;
    end
end


if (strcmp(algtype,'LASSO') || strcmp(algtype,'SP') || strcmp(algtype,'OMP'))
    [tmp,indx] = min(nmse_x_test);
else
    tmp = (norm(xhat-x)^2)/(norm(x)^2);
end

nmse(i) = tmp;

end;

%save the results
date = datestr(now, 'ddmmmyyyy_HHMMSSFFF');
savefile = [strcat('SNRphase1D_', algtype) num2str(date) '.mat'];

save(savefile, 'nmse', 'SNR')

toc(tstart)
clear all