function SPARSEphase1D(algtype, signaltype)
%SPARSEphas1D serves as a test of sparse signal recovery. It cycles through
%the number of measurements M for a fixed signal dimension N
%
%
%SYNTAX:
%SPARSEphas1D(algtype, signaltype)
%
%INPUTS:
% algtype - which algorithm to run
% - 'EMBGAMP'
% - 'EMGMAMP'
% - 'EMGMAMPMOS'
% - 'SL0' Smoothed L0 norm
% - 'BCS' Bayesian Compressive Sensing
% - 'SBL' Sparse bayesian laearning via TMSBL
% - 'LASSO' LASSO via SPGL-1
% - 'OMP' Orhtongonal Matching Pursuit via Solve OMP
% - 'SP' Subspace Pursuit
% signaltype - which type of signal
% -'BG' Bernoulli Gaussian
% -'BR' Bernoulli Rademacher
% -'B1' Bernoulli
%
%Outputs
%The dated .mat file will containing the nmse for each gridpoint.  
%It also contains the CPU time for the algorithm
%
%Coded by Jeremy Vila
%The Ohio State University
%vila.2@osu.edu
%Date 7-01-13
%Version 1.0

%define seed based on internal clock
tstart2 = tic;
%define seed based on internal clock
NewStream = RandStream.create('mt19937ar', 'Seed', sum(10000*clock));
% RandStream.setDefaultStream(NewStream);
% savedState = NewStream.State;

%Handle random seed
if verLessThan('matlab','7.14')
  RandStream.setDefaultStream(NewStream);
else
  RandStream.setGlobalStream(NewStream);
end;

%size of signal
N = 1000;
%SNR in dB.
SNRdB = 25;
%amount of undersampling
rows = 200:20:500;

%initialize the nmse and time vectors
nmse = zeros(1,length(rows));
time = zeros(1,length(rows));
for i = 1:length(rows)

%Define number of measurements.
M = rows(i);

%A matrix, Gaussian
Amat = randn(M,N);

%Normalize the columns
columnNorms = sqrt(diag(Amat'*Amat));
Amat = Amat*diag(1 ./ columnNorms); %unit norm columns

%Make A an operator for GAMP
A = MatrixLinTrans(Amat); 

%Sparsity
K = 100;

truebits = false(N,1);
truebits(1:K) = true; %which bits are on

% Generate the signal
%Bernoulli Gaussian
if strcmp(signaltype,'BG')
    x = randn(N,1).* truebits;
%Bernoulli Rademacher
elseif strcmp(signaltype,'BR');
    x = sign(randn(N,1)).* truebits;
%Bernoulli 
elseif strcmp(signaltype,'B1');
    x = ones(N,1).* truebits;
end
%Randomly locate the active elements
x = x(randperm(length(x)));

%Generate noiseless measurements
ztrue = A.mult(x);

%Calculate noise variance
muw(i) = norm(A.mult(x))^2/M*10^(-SNRdB/10);

%Compute noisy output
y = ztrue + sqrt(muw(i))*(randn(M,1));

%start clock
tstart = tic;
%EMBGAMP
if strcmp(algtype,'EMBGAMP')
    optEM.heavy_tailed = false;
    xhat = EMBGAMP(y, A, optEM);
%EMGMAMP
elseif strcmp(algtype,'EMGMAMP')
    optEM.heavy_tailed = false;
    xhat = EMGMAMP(y, A, optEM);
elseif strcmp(algtype,'EMGMAMPMOS')
    optEM.heavy_tailed = false;
    xhat = EMGMAMPMOS(y, A, optEM);
%SL0 algorithm
elseif strcmp(algtype,'SL0')
    sigma_min = 0.00004;
    xhat = SL0(Amat, y, sigma_min);
%Bayesian Compressive sensing algorithm BCS
elseif strcmp(algtype,'BCS')
    xhat = mt_CS(Amat,y,100/var(y),1,1e-8);
%Sparse Bayesian Learning programmed by Zhiling Zhang
elseif strcmp(algtype,'SBL')
    if SNRdB < 22
        xhat = TMSBL(Amat, y,'noise','mild','PRINT',0);
    else
        xhat = TMSBL(Amat, y,'noise','small','PRINT',0); 
    end
%LASSO via SPGL-1
elseif strcmp(algtype,'LASSO')
    options.verbosity = 0;
    %Create grid of "lambda values"
    scale = [0.6:0.2:1.4];
    %Run SPGL-1 for each value on the grid and save nmse for each test
    for v=1:length(scale),      % find nmse-optimal error constraint
         xhat = spgl1(Amat,y,0,sqrt(scale(v)*muw(i)*M),[],options);
         %Debias the estimates
         A_debiased = Amat(:,(xhat~=0));
         xhat_debiased = zeros(N,1);
         xhat_debiased((xhat~=0)) = A_debiased\y;
         nmse_x_test(v) = norm(xhat_debiased-x)^2/norm(x)^2;
    end
%OMP
elseif strcmp(algtype,'OMP')
    %Generate grid of postulated K
    iters = floor(linspace(1,2*K,10));
    %Perform OMP for this grid and save the nmse for each test
    for j = 1:length(iters)
        xhat = SolveOMP(Amat, y, N,iters(j));
        nmse_x_test(j) = norm(xhat-x)^2/norm(x)^2;
    end  
%Subspace pursuit
elseif strcmp(algtype,'SP')
    %Generate grid of postulated K
    iters = floor(linspace(1,2*K,10));
    %Perform SP for this grid and save the nmse for each test
    for j = 1:length(iters)
        tmp = CSRec_SP(iters(j), Amat, y);
        xhat = tmp.x_hat;
        nmse_x_test(j) = norm(xhat-x)^2/norm(x)^2;
    end
end

%Get the best NMSE for LASSO,SP,or OMP
if (strcmp(algtype,'LASSO') || strcmp(algtype,'SP') || strcmp(algtype,'OMP'))
    [tmp,indx] = min(nmse_x_test);
else
    tmp = (norm(xhat-x)^2)/(norm(x)^2);
end

%save nmse and time
nmse(i) = tmp;
time(i) = toc(tstart);

end;

%save the results
date = datestr(now, 'ddmmmyyyy_HHMMSSFFF');
savefile = [strcat('SPASEphas1D_',algtype,'_',signaltype,'_') date '.mat'];

save(savefile, 'nmse', 'time', 'rows')

toc(tstart2)
return