function TIMEphase1DGaus(logNmax, algtype, Tmax)
%TIMEphase1DGaus serves as a test of the sparse Bernoulli Rademacher signal
%recovery. For a fixed undersampling ratio M/N = 0.5, it grows the
%dimension on a log scale.  The i.i.d. Gaussian matrix is used as the 
%sensing operator A.
%
%
%SYNTAX:
%TIMEphase1DGaus(gridsize,logNmax, SNRdB,Tmax)
%
%INPUTS:
%logNmax            maximum value of log10(N).
%algtype            type of algortithm used
% - 'EMBGAMP'
% - 'EMGMAMP'
% - 'EMGMAMPMOS'
% - 'SL0' Smoothed L0 norm
% - 'BCS' Bayesian Compressive Sensing
% - 'SBL' Sparse bayesian laearning via TMSBL
% - 'LASSO' LASSO via SPGL-1
% - 'OMP' Orhtongonal Matching Pursuit via Solve OMP
% - 'SP' Subspace Pursuit
%Tmax               Set maximum time value for a single recovery instance
%
%Outputs
%The dated .mat file will contain the nmse for each gridpoint.
%It also contains the CPU time for the algorithm and the grid of signal 
%dimension N used.
%
%Coded by Jeremy Vila
%The Ohio State University
%vila.2@osu.edu
%Date 7-01-13
%Version 1.0

tstart = tic;
%define seed based on internal clock
NewStream = RandStream.create('mt19937ar', 'Seed', sum(10000*clock));
% RandStream.setDefaultStream(NewStream);
% savedState = NewStream.State;

%Handle random seed
if verLessThan('matlab','7.14')
  RandStream.setDefaultStream(NewStream);
else
  RandStream.setGlobalStream(NewStream);
end;

gridsize = 11;
Naxis = logspace(3,logNmax,gridsize);
nmse = Inf*ones(1,gridsize);
time = Inf*ones(1,gridsize);
SNRdB = 25;

%Preallocate memory
nmse = Inf*ones(1,gridsize);
time = Inf*ones(1,gridsize);

stop = false;
t = 1;

while ~stop

if t >= length(Naxis)
    stop = true;
end

%Find problem dimensions for each iteration
N = ceil(Naxis(t));
M = ceil(.5*N);
K = floor(.1*N);

%Avoid zero K
if K == 0
    K = 1;
end

%Determine true bits
truebits = false(N,1);
truebits(1:K) = true; %which bits are on

%Compute true input vector
x = sign(randn(N,1)).* truebits;
x = x(randperm(length(x)));

% Build the Gaussian matrix
Amat = randn(M,N);

columnNorms = sqrt(diag(Amat'*Amat));
Amat = Amat*diag(1 ./ columnNorms); %unit norm columns

%Rename A
A = MatrixLinTrans(Amat);

%Form noiseless measurements
ztrue = A.mult(x);

muw(t) = norm(ztrue)^2/M*10^(-SNRdB/10);

%Compute noisy output
y = ztrue + sqrt(muw(t)/2)*(randn(M,1)+1i*randn(M,1));

tstart2 = tic;
%EMGMAMPMOS
if strcmp(algtype,'EMGMAMPMOS')
    optEM.heavy_tailed = false;
    optEM.cmplx_in = false;
    optEM.cmplx_out = true;
    xhat = EMGMAMPMOS(y, A, optEM);
%EMGMAMP
elseif strcmp(algtype,'EMGMAMP')
    optEM.heavy_tailed = false;
    optEM.cmplx_in = false;
    optEM.cmplx_out = true;
    xhat = EMGMAMP(y, A, optEM);
%EMBGAMP
elseif strcmp(algtype,'EMBGAMP')
    optEM.heavy_tailed = false;
    optEM.cmplx_in = false;
    optEM.cmplx_out = true;
    xhat = EMBGAMP(y, A, optEM);
%SL0 algorithm
elseif strcmp(algtype,'SL0')
    sigma_min = 0.00004;
    xhat = SL0(Amat, y, sigma_min);
%Bayesian Compressive sensing algorithm BCS
elseif strcmp(algtype,'BCS')
    xhat = mt_CS(Amat,y,100/var(y),1,1e-8);
%Sparse Bayesian Learning programmed by Zhiling Zhang
elseif strcmp(algtype,'SBL')
    if SNRdB < 22
        xhat = TMSBL(Amat, y,'noise','mild','PRINT',0);
    else
        xhat = TMSBL(Amat, y,'noise','small','PRINT',0); 
    end
%LASSO
elseif strcmp(algtype,'LASSO')
    options.verbosity = 0;
    scale = [0.3:0.2:1.1];
    for v=1:length(scale),      % find nmse-optimal error constraint
     x_hat_test = spgl1(Amat,y,0,sqrt(scale(v)*norm(y)^2/1001),[],options);
     nmse_x_test(v) = norm(x_hat_test-x)^2/norm(x)^2;
    end;
%OMP
elseif strcmp(algtype,'OMP')
    %Generate grid of postulated K
    iters = floor(linspace(1,2*K,10));
    %Perform OMP for this grid and save the nmse for each test
    for j = 1:length(iters)
        xhat = SolveOMP(Amat, y, N,iters(j));
        nmse_x_test(j) = norm(xhat-x)^2/norm(x)^2;
    end  
%Subspace pursuit
elseif strcmp(algtype,'SP')
    %Generate grid of postulated K
    iters = floor(linspace(1,2*K,10));
    %Perform SP for this grid and save the nmse for each test
    for j = 1:length(iters)
        tmp = CSRec_SP(iters(j), Amat, y);
        xhat = tmp.x_hat;
        nmse_x_test(j) = norm(xhat-x)^2/norm(x)^2;
    end
end

time(t) = toc(tstart2);

%Get the best NMSE for LASSO,SP,or OMP
if (strcmp(algtype,'LASSO') || strcmp(algtype,'SP') || strcmp(algtype,'OMP'))
    [tmp,~] = min(nmse_x_test);
else
    tmp = (norm(xhat-x)^2)/(norm(x)^2);
end

nmse(t) = tmp;

if time(t) > Tmax
    stop = true;
end

t = t+1;

end;

%save the results
date = datestr(now, 'ddmmmyyyy_HHMMSSFFF');
savefile = [strcat('TIMEphase1DGaus_',algtype,'_') date '.mat'];

save(savefile, 'time', 'nmse', 'Naxis','Tmax')

toc(tstart)
clear all