function TIMEphase1Dfft(algtype, Tmax)
%TIMEphase1Dfft serves as a test of the sparse Bernoulli Rademacher signal
%recovery. For a fixed undersampling ratio M/N = 0.5, it grows the
%dimension on a log scale.  The fft is used as the sensing operator A.
%
%
%SYNTAX:
%TIMEphase1Dfft(algtype, Tmax)
%
%INPUTS:
%algtype 
% - 'EMBGAMP'
% - 'EMGMAMP'
% - 'EMGMAMPMOS'
% - 'LASSO' LASSO via SPGL-1
%Tmax               Set maximum time value for a single recovery instance
%
%Outputs
%The dated .mat file will contain the nmse for each gridpoint.
%It also contains the CPU time for the algorithm and the grid of signal 
%dimension N used.
%
%Coded by Jeremy Vila
%The Ohio State University
%vila.2@osu.edu
%Date 7-01-13
%Version 1.0

tstart = tic;
%define seed based on internal clock
NewStream = RandStream.create('mt19937ar', 'Seed', sum(10000*clock));
% RandStream.setDefaultStream(NewStream);
% savedState = NewStream.State;

%Handle random seed
if verLessThan('matlab','7.14')
  RandStream.setDefaultStream(NewStream);
else
  RandStream.setGlobalStream(NewStream);
end;

%Define the axis
Naxis = 10:20;
Naxis = 2.^Naxis;
L = length(Naxis);
SNRdB = 25;

%Preallocate memory
nmse = Inf*ones(1,L);
time = Inf*ones(1,L);

stop = false;
t = 1;

while ~stop

if t >= length(Naxis)
    stop = true;
end

%Find problem dimensions for each iteration
N = ceil(Naxis(t));
M = ceil(.5*N);
K = floor(.1*N);

%Avoid zero K
if K == 0
    K = 1;
end

%Determine true bits
truebits = false(N,1);
truebits(1:K) = true; %which bits are on

%Compute true input vector
x = sign(randn(N,1)).* truebits;
x = x(randperm(length(x)));

% Build the fft operator
if (strcmp(algtype,'EMBGAMP') || strcmp(algtype,'EMGMAMP') ...
        || strcmp(algtype,'EMGMAMPMOS'))
    % fft for GAMP employing functions
    A = FourierLinTrans(N,N,1);

    %Compute random sample locations
    A.ySamplesRandom(M);
    
    %Calcualte noiseless measurements
    ztrue = A.mult(x);
elseif strcmp(algtype,'LASSO')
    % fft for SPGL1
    idx = randperm(N); idx = idx(1:M);
    opA = @(x,mode) partialFourier(idx,N,x,mode);
    ztrue = opA(x,1);
end

muw(t) = norm(ztrue)^2/M*10^(-SNRdB/10);

%Compute noisy output
y = ztrue + sqrt(muw(t)/2)*(randn(M,1)+1i*randn(M,1));

tstart2 = tic;
if strcmp(algtype,'EMGMAMPMOS')
    optEM.heavy_tailed = false;
    optEM.cmplx_in = false;
    optEM.cmplx_out = true;
    xhat = EMGMAMPMOS(y, A, optEM);
elseif strcmp(algtype,'EMGMAMP')
    optEM.heavy_tailed = false;
    optEM.cmplx_in = false;
    optEM.cmplx_out = true;
    xhat = EMGMAMP(y, A, optEM);
elseif strcmp(algtype,'EMBGAMP')
    optEM.heavy_tailed = false;
    optEM.cmplx_in = false;
    optEM.cmplx_out = true;
    xhat = EMBGAMP(y, A, optEM);
elseif strcmp(algtype,'LASSO')
    options.verbosity = 0;
    scale = [0.3:0.2:1.1];
    for v=1:length(scale),      % find nmse-optimal error constraint
     x_hat_test = spgl1(opA,y,0,sqrt(scale(v)*norm(y)^2/1001),[],options);
     nmse_x_test(v) = norm(x_hat_test-x)^2/norm(x)^2;
    end;
end

time(t) = toc(tstart2);

%Get the best NMSE for LASSO,SP,or OMP
if strcmp(algtype,'LASSO')
    [tmp,~] = min(nmse_x_test);
else
    tmp = (norm(xhat-x)^2)/(norm(x)^2);
end

nmse(t) = tmp;

if time(t) > Tmax
    stop = true;
end

t = t+1;

end;

%save the results
date = datestr(now, 'ddmmmyyyy_HHMMSSFFF');
savefile = [strcat('TIMEphase1Dfft_',algtype,'_') date '.mat'];

save(savefile, 'time', 'nmse', 'Naxis','Tmax')

toc(tstart)
clear all