function audio_Gaus(algtype,N)
%audio_Gaus serves as a test of recovery of an audio file from compressed
%measurements.  Here, an i.i.d. Gaussian matrix is used as the sensing
%operator 'Phi" (see EMGMAMP paper for details)
%
%
%SYNTAX:
%audio_Gaus(algtype,N)
%
%INPUTS:
% algtype - which algorithm to run
% - 'EMBGAMP'
% - 'EMGMAMP'
% - 'EMGMAMPMOS'
% - 'SL0' Smoothed L0 norm
% - 'BCS' Bayesian Compressive Sensing
% - 'SBL' Sparse bayesian laearning via TMSBL
% - 'LASSO' LASSO via SPGL-1
% - 'OMP' Orhtongonal Matching Pursuit via Solve OMP
% - 'SP' Subspace Pursuit
% N length of signal.  Depending on this length, the total number of blocks
% will be different.
%
%Outputs
%The dated .mat file will containing the nmse and cpu time using the chosen
%algorithm.
%
%Coded by Jeremy Vila
%The Ohio State University
%vila.2@osu.edu
%Date 7-01-13
%Version 1.0

tstart_tot = tic;

NewStream = RandStream.create('mt19937ar', 'Seed', sum(10000*clock));
% RandStream.setDefaultStream(NewStream);
% savedState = NewStream.State;

%Handle random seed
if verLessThan('matlab','7.14')
  RandStream.setDefaultStream(NewStream);
else
  RandStream.setGlobalStream(NewStream);
end;

%Read the audio file
[f,fs] = wavread('moz1_11kHz.wav');

fx=buffer(f,N);
M=ceil(1/2*N);

%Build Gaussian matrix
Phi = randn(M,N);

%Normalize the columns
columnNorms = sqrt(diag(Phi'*Phi));
Phi = Phi*diag(1 ./ columnNorms); %unit norm columns

%Build full operator
Amat=zeros(M,N);
for k=1:M
Amat(k,:)=dct(Phi(k,:))';
end
[w,T] = size(fx);
f_est = zeros(w,T);
tmp = zeros(T,1);

Y = Phi*fx;

%Define cdf/pdf for Gaussian
normal_cdf = @(x) 1/2*(1 + erf(x/sqrt(2)));

%Define density of Gaussian
normal_pdf = @(x) 1/sqrt(2*pi)*exp(-x.^2/2);

delta = M/N;
temp = linspace(0,100,1024);
rho_SE = (1 - (2/delta)*((1+temp.^2).*normal_cdf(-temp)-temp.*normal_pdf(temp)))./(1 + temp.^2 - 2*((1+temp.^2).*normal_cdf(-temp)-temp.*normal_pdf(temp)));
rho = max(rho_SE);

tstart = tic;
%tstart = tic;
for i = 1:T
    %EMBGAMP using sourceforge
    if strcmp(algtype,'EMBGAMP')
        optEM.heavy_tailed = true;
        optEM.maxEMiter = 50;
        xhat = EMBGAMP(Y(:,i), Amat, optEM);
    %EMGMAMP using sourceforge
    elseif strcmp(algtype,'EMGMAMP')
        optEM.heavy_tailed = true;
        optEM.maxEMiter = 50;
        xhat = EMGMAMP(Y(:,i), Amat, optEM);
    elseif strcmp(algtype,'EMGMAMPMOS')
        optEM.heavy_tailed = true;
        optEM.maxEMiter = 50;
        xhat = EMGMAMPMOS(Y(:,i), Amat, optEM);
    %SL0 algorithm
    elseif strcmp(algtype,'SL0')
        %sigma_min = sqrt(muw(t))*3;
        sigma_min = 0.00004;
        xhat = SL0(Amat, Y(:,i), sigma_min);
    %Bayesian Compressive sensing algorithm BCS
    elseif strcmp(algtype,'BCS')
        xhat = mt_CS(Amat,Y(:,i),100/var(Y(:,i)),1,1e-8);
    elseif strcmp(algtype,'SBL')
         xhat = TMSBL(Amat, Y(:,i),'prune_gamma',1e-12,'learn_lambda',0,'lambda',1e-15,'PRINT',0); 
    elseif strcmp(algtype,'LASSO')
    %LASSO via SPGL-1
        options.verbosity = 0;
        scale = [0.6:0.2:1.4];
        for j = 1:length(scale)
            xhat = spgl1(Amat,Y(:,i),0,sqrt(scale(j)*1e-6*M),[],options);
            nmse_x_test(j) = norm(fx(:,i)-idct(xhat))^2/norm(fx(:,i))^2;
        end
    %OMP
    elseif strcmp(algtype,'OMP')
        iters = floor(linspace(1,floor(rho*M),10));
        for j = 1:length(iters)
            if j == length(iters)
                tstart = tic;
            end;
            xhat = SolveOMP(Amat, Y(:,i), N,iters(j));
            if j == length(iters)
                time = time + toc(tstart);
            end
            nmse_x_test(j) = norm(fx(:,i)-idct(xhat))^2/norm(fx(:,i))^2;
        end 
    %SP
    elseif strcmp(algtype,'SP')
        iters = floor(linspace(1,floor(rho*M),10));
        for j = 1:length(iters)
            dummy = CSRec_SP(iters(j), Amat, Y(:,i));
            xhat = dummy.x_hat;
            nmse_x_test(j) = norm(fx(:,i)-idct(xhat))^2/norm(fx(:,i))^2;
        end
    end


    if (strcmp(algtype,'LASSO') || strcmp(algtype,'SP') || strcmp(algtype,'OMP'))
        [tmp(i),indx] = min(nmse_x_test);
    else
        f_est(:,i)=idct(xhat);
        tmp(i) = norm(fx(:,i)-f_est(:,i))^2/norm(fx(:,i))^2;
    end
   
end
time = toc(tstart);
nmse = tmp;
10*log10(mean(nmse));

%save the results
date = datestr(now, 'ddmmmyyyy_HHMMSSFFF');
savefile = [strcat('audio_Gaus_', algtype,'_') num2str(date) '.mat'];

save(savefile, 'nmse', 'time')

toc(tstart_tot)