function audio_SEL(algtype,N)
%audio_SEL serves as a test of recovery of an audio file from compressed
%measurements.  Here, a selectiion matrix is used as the sensing
%operator 'Phi" (see EMGMAMP paper for details)
%
%
%SYNTAX:
%audio_SEL(algtype,N)
%
%INPUTS:
% algtype - which algorithm to run
% - 'EMBGAMP'
% - 'EMGMAMP'
% - 'EMGMAMPMOS'
% - 'SL0' Smoothed L0 norm
% - 'BCS' Bayesian Compressive Sensing
% - 'SBL' Sparse bayesian laearning via TMSBL
% - 'LASSO' LASSO via SPGL-1
% - 'OMP' Orhtongonal Matching Pursuit via Solve OMP
% - 'SP' Subspace Pursuit
% N length of signal.  Depending on this length, the total number of blocks
% will be different.
%
%Outputs
%The dated .mat file will containing the nmse and cpu time using the chosen
%algorithm.
%
%Coded by Jeremy Vila
%The Ohio State University
%vila.2@osu.edu
%Date 7-01-13
%Version 1.0

tstart_tot = tic;

NewStream = RandStream.create('mt19937ar', 'Seed', sum(10000*clock));
% RandStream.setDefaultStream(NewStream);
% savedState = NewStream.State;

%Handle random seed
if verLessThan('matlab','7.14')
  RandStream.setDefaultStream(NewStream);
else
  RandStream.setGlobalStream(NewStream);
end;

%Read the audio file
[f,fs] = wavread('moz1_11kHz.wav');

fx=buffer(f,N);
M=ceil(1/2*N);

if (strcmp(algtype,'EMBGAMP') || strcmp(algtype,'EMGMAMP') || ...
        strcmp(algtype,'EMGMAMPMOS'))
    %Build explicit operator for GAMP
    A = DCTLinTrans(N,N,1);

    whichRows = randperm(N);
    whichRows = whichRows(1:M);

    A.ySamplesSetFromSubScripts(whichRows');
elseif strcmp(algtype,'LASSO')
    %dct for SPGL1
    whichRows = randperm(N); 
    whichRows = whichRows(1:M);
    opA = @(x,mode) partialDCT(whichRows,N,x,mode);
else
    %explicit "wide" DCT
    A = dctmtx(N);
    whichRows = randperm(N); 
    whichRows = whichRows(1:M);
    A = A(whichRows,:);
end

%get problem dimensions
[w,T] = size(fx);
f_est = zeros(w,T);
tmp = zeros(T,1);

%Subsample the measurements
Y = fx(whichRows,:);

%Define cdf/pdf for Gaussian
normal_cdf = @(x) 1/2*(1 + erf(x/sqrt(2)));

%Define density of Gaussian
normal_pdf = @(x) 1/sqrt(2*pi)*exp(-x.^2/2);

delta = M/N;
temp = linspace(0,100,1024);
rho_SE = (1 - (2/delta)*((1+temp.^2).*normal_cdf(-temp)-temp.*normal_pdf(temp)))./(1 + temp.^2 - 2*((1+temp.^2).*normal_cdf(-temp)-temp.*normal_pdf(temp)));
rho = max(rho_SE);

tstart = tic;
for i = 1:T
    %EMBGAMP using sourceforge
    if strcmp(algtype,'EMBGAMP')
        optEM.heavy_tailed = true;
        optEM.maxEMiter = 50;
        xhat = EMBGAMP(Y(:,i), A, optEM);
    %EMGMAMP using sourceforge
    elseif strcmp(algtype,'EMGMAMP')
        optEM.heavy_tailed = true;
        optEM.maxEMiter = 50;
        xhat = EMGMAMP(Y(:,i), A, optEM);
    elseif strcmp(algtype,'EMGMAMPMOS')
        optEM.heavy_tailed = true;
        optEM.maxEMiter = 50;
        xhat = EMGMAMPMOS(Y(:,i), A, optEM);
    %SL0 algorithm
    elseif strcmp(algtype,'SL0')
        %sigma_min = sqrt(muw(t))*3;
        sigma_min = 0.0004;
        xhat = SL0(A, Y(:,i), sigma_min);
    %BCS
    elseif strcmp(algtype,'BCS')
        xhat = mt_CS(A,Y(:,i),100/var(Y(:,i)),1,1e-8);
    %SBL
    elseif strcmp(algtype,'SBL')
         xhat = TMSBL(A, Y(:,i),'prune_gamma',1e-12,'learn_lambda',0,'lambda',1e-15,'PRINT',0); 
    elseif strcmp(algtype,'LASSO')
    %LASSO via SPGL-1
        options.verbosity = 0;
        scale = [0.6:0.2:1.4];
        for j = 1:length(scale)
            xhat = spgl1(opA,Y(:,i),0,sqrt(scale(j)*1e-6*M),[],options);
            nmse_x_test(j) = norm(fx(:,i)-dct(xhat))^2/norm(fx(:,i))^2;
        end
    %OMP
    elseif strcmp(algtype,'OMP')
        iters = floor(linspace(1,floor(rho*M),10));
        for j = 1:length(iters)
            if j == length(iters)
                tstart = tic;
            end;
            xhat = SolveOMP(A, Y(:,i), N,iters(j));
            nmse_x_test(j) = norm(fx(:,i)-dct(xhat))^2/norm(fx(:,i))^2;
        end 
    %SP
    elseif strcmp(algtype,'SP')
        iters = floor(linspace(1,floor(rho*M),10));
        for j = 1:length(iters)
            dummy = CSRec_SP(iters(j), A, Y(:,i));
            xhat = dummy.x_hat;
            nmse_x_test(j) = norm(fx(:,i)-dct(xhat))^2/norm(fx(:,i))^2;
        end
    end


    if (strcmp(algtype,'LASSO') || strcmp(algtype,'SP') || strcmp(algtype,'OMP'))
        [tmp(i),indx] = min(nmse_x_test);
    else
        f_est(:,i)= dct(xhat);
        tmp(i) = norm(fx(:,i)-f_est(:,i))^2/norm(fx(:,i))^2;
    end
   
end
time = toc(tstart);
nmse = tmp;

%save the results
date = datestr(now, 'ddmmmyyyy_HHMMSSFFF');
savefile = [strcat('audio_SEL_',num2str(N),'_') num2str(date) '.mat'];

save(savefile, 'nmse', 'time')

toc(tstart_tot)