function logPTC(algtype)
%logPTC serves as a test of the sparse Bernoulli Gaussian signal
%recovery. It cycles through the undersampling ratio (m/n),delta, and the
%active coefficient to measurement ratio (lambda n/m), rho. 
%
%
%SYNTAX:
%LOGPTC(algtype)
%
%INPUTS:
%algtype         type of algorithm to run
%
%Outputs
%The dated .mat file will contain a matrix of nmse for each gridpoint.
%
%Coded by Jeremy Vila
%The Ohio State University
%vila.2@osu.edu
%Date 7-01-13
%Version 1.0

tic

%define seed based on internal clock
NewStream = RandStream.create('mt19937ar', 'Seed', sum(10000*clock));
% RandStream.setDefaultStream(NewStream);
% savedState = NewStream.State;

%Handle random seed
if verLessThan('matlab','7.14')
  RandStream.setDefaultStream(NewStream);
else
  RandStream.setGlobalStream(NewStream);
end;

%Define signal dimension and SNR
N = 8192;
SNRdB = Inf;

%Define log axes
M_axis = 2.^(3:1:log2(N)-4);
K_axis = 2.^(0:1:log2(N)-4);
M_length = length(M_axis);
K_length = length(K_axis);

nmse = ones(K_length,M_length);

for m = 1:M_length
for k = 1:K_length
    
wdw = 0;

M = M_axis(m);
K = K_axis(k);

if strcmp(algtype,'FISTA')
    Amat = dctmtx(N);
    whichRows = randperm(N); whichRows = whichRows(1:M);
    Amat = Amat(whichRows,:);

    %Rename A
    A = MatrixLinTrans(Amat);
else
    A = DCTLinTrans(N,N,1);

    %Compute random sample locations
    A.ySamplesRandom(M);
end

%Determine true bits
truebits = false(N,1);
truebits(1:K) = true; %which bits are on

%Compute true input vector
x = randn(N,1).* truebits;
%randomize elements of x
x = x(randperm(N));

ztrue = A.mult(x);

%Output channel- we assume AWGN with mean thetahat and variance muw
muw = norm(ztrue)^2/M*10^(-SNRdB/10);

%Compute noisy output

y = ztrue + sqrt(muw)*(randn(M,1));

if strcmp(algtype,'EMGMAMP')
    optEM.EMtol = 1e-6;
    optEM.maxEMiter = 50;
    optEM.noise_var = 1e-8;
    optEM.heavy_tailed = false;
    optEM.learn_noisevar = true;
    optGAMP = GampOpt();
    optGAMP.nit = 50;
    optGAMP.tol = 1e-7;
    [xhat, ~, ~] = EMGMAMP(y, A, optEM ,optGAMP);
elseif strcmp(algtype,'EMBGAMP')
    optEM.EMtol = 1e-6;
    optEM.maxEMiter = 50;
    optEM.noise_var = 1e-8;
    optEM.heavy_tailed = false;
    optEM.learn_noisevar = true;
    optGAMP = GampOpt();
    optGAMP.nit = 50;
    optGAMP.tol = 1e-7;
    [xhat, ~, ~] = EMBGAMP(y, A, optEM , optGAMP);
elseif strcmp(algtype,'genieAMP')   
    %Set options for Rangan GAMP
    optGAMP = GampOpt(); %initialize the options object
    optGAMP.nit = 2000;
    optGAMP.adaptStep = false;
    optGAMP.verbose = false;
    optGAMP.removeMean = false;
    optGAMP.pvarMin = 0;
    optGAMP.xvarMin = 0;
    optGAMP.tol = 1e-7;
    
    %Create the input estimator
    inputEst = AwgnEstimIn(0, 1);  %make the scalar estimator
    inputEst = SparseScaEstim(inputEst,K/N); %construct Gin for Bernouli

    %Output channel
    outputEst = AwgnEstimOut(y, muw);

    %Perform genie AMP
    [xhat, ~, ~, ~,...
        ~, ~, ~] = ...
        gampEst(inputEst, outputEst, A, optGAMP);
elseif strcmp(algtype,'FISTA')
    FISTAopt = FistaOpt();
    FISTAopt.tol = 1e-5; %same tolerance
    FISTAopt.nit = 10000; %but often more iterations
    FISTAopt.verbose = false;
    FISTAopt.lam = 1e-5;
    
          [resFISTA,estHistFISTA] = fistaEst(...
          zeros(size(x)),y,A,FISTAopt);
      xhat = estHistFISTA.xhat(:,end);
      
elseif strcmp(algtype,'EMGMAMPMOS')
    optEM.EMtol = 1e-6;
    optEM.maxEMiter = 50;
    optEM.noise_var = 1e-8;
    optEM.heavy_tailed = false;
    optEM.learn_noisevar = true;
    optGAMP = GampOpt();
    optGAMP.nit = 50;
    optGAMP.tol = 1e-7;
    xhat = EMGMAMPMOS(y, A, optEM, optGAMP);

end

nmse(k,m) = (norm(xhat-x)^2)/(norm(x)^2);

if 10*log10(nmse(k,m)) > -5
    wdw = wdw + 1;
end;

if wdw > 1
    break
end

end
wdw = 0;
end

date = datestr(now, 'ddmmmyyyy_HHMMSSFFF');
savefile = [strcat('LOGPTC_',algtype,'_') num2str(date) '.mat'];

save(savefile, 'nmse', 'M_axis', 'K_axis', 'N')
toc